@@ -15,6 +15,7 @@
 ##'     greater, multiple control streams will be generated.
 ##' @import NMdata
 ##' @import data.table
+##' @importFrom MASS mvrnorm
 ##' @return Character vector of simulation control stream paths
 ##' @export
 
@@ -63,9 +64,11 @@ NMsim_VarCov <- function(file.sim,file.mod,data.sim,nsims=1){
     
     ## nonmem2rx::nmcov(path.cov)
     covmat <- NMreadCov(path.cov)
-    ests <- NMreadExt(path.ext)$pars[NMREP==1,.(parameter,par.type,i,j,est,FIX)]
+    ests <- NMdata::NMreadExt(path.ext,as.fun="data.table")[NMREP==1,.(parameter,par.type,i,j,est,FIX)]
+    ests <- ests[par.type%in%c("THETA","SIGMA","OMEGA")]
+    ## ests <- NMreadExt(path.ext)$pars[NMREP==1,.(parameter,par.type,i,j,est,FIX)]
     ests <- ests[match(ests$parameter,colnames(covmat))]
-
+    
     
     newpars <- mvrnorm(n=nsims,Sigma=covmat,mu=ests$est)
     ### as.list first is because without it, this will fail for
